@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 # gPodder - A media aggregator and podcast client
-# Copyright (c) 2005-2012 Thomas Perl and the gPodder Team
+# Copyright (c) 2005-2011 Thomas Perl and the gPodder Team
 #
 # gPodder is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,12 +18,14 @@
 #
 
 __author__    = 'Thomas Perl <thp@gpodder.org>'
-__version__   = '2.20.2'
-__date__      = '2012-09-24'
-__copyright__ = '© 2005-2012 Thomas Perl and the gPodder Team'
-__licence__   = 'GNU General Public License, version 3 or later'
+__version__   = '3.0.1'
+__date__      = '2011-11-13'
+__copyright__ = '© 2005-2011 Thomas Perl and the gPodder Team'
+__license__   = 'GNU General Public License, version 3 or later'
 __url__       = 'http://gpodder.org/'
 
+__version_info__ = tuple(int(x) for x in __version__.split('.'))
+
 import os
 import sys
 import platform
@@ -35,7 +37,7 @@ try:
     import feedparser
 except ImportError:
     print """
-  Error: Module "feedparser" not found. Please install "python-feedparser".
+  Error: Module "feedparser" (python-feedparser) not found.
          The feedparser module can be downloaded from
          http://code.google.com/p/feedparser/
 """
@@ -46,8 +48,9 @@ try:
     import mygpoclient
 except ImportError:
     print """
-  Error: Module "mygpoclient" not found. Please install "python-mygpoclient"
-         or download it from http://thp.io/2010/mygpoclient/
+  Error: Module "mygpoclient" (python-mygpoclient) not found.
+         The mygpoclient module can be downloaded from
+         http://thp.io/2010/mygpoclient/
 """
     sys.exit(1)
 del mygpoclient
@@ -59,12 +62,11 @@ user_agent = 'gPodder/%s (+%s)' % (__version__, __url__)
 class UI(object):
     def __init__(self):
         self.desktop = False
-        self.diablo = False
         self.fremantle = False
+        self.harmattan = False
+        self.qt = False
+        self.gtk = False
 
-    @property
-    def maemo(self):
-        return self.diablo or self.fremantle
 
 ui = UI()
 
@@ -78,6 +80,8 @@ dbus_session_bus = None
 
 # Set "win32" to True if we are on Windows
 win32 = (platform.system() == 'Windows')
+# Set "osx" to True if we are on Mac OS X
+osx = (platform.system() == 'Darwin')
 
 # i18n setup (will result in "gettext" to be available)
 # Use   _ = gpodder.gettext   in modules to enable string translations
@@ -87,23 +91,19 @@ t = gettext.translation(textdomain, locale_dir, fallback=True)
 gettext = t.ugettext
 ngettext = t.ungettext
 if win32:
-    # Workaround for bug 650
-    from gtk.glade import bindtextdomain
-    bindtextdomain(textdomain, locale_dir)
-    del bindtextdomain
+    try:
+        # Workaround for bug 650
+        from gtk.glade import bindtextdomain
+        bindtextdomain(textdomain, locale_dir)
+        del bindtextdomain
+    except:
+        # Ignore for QML UI or missing glade module
+        pass
 del t
 
 # Set up textdomain for gtk.Builder (this accesses the C library functions)
 if hasattr(locale, 'bindtextdomain'):
     locale.bindtextdomain(textdomain, locale_dir)
-else:
-    # On Win32, the locale module does not have bindtextdomain. We use a
-    # small module that provides similar functionality here (from doc/dev/).
-    try:
-        import gtkbuilderi18n
-        gtkbuilderi18n.bindtextdomain(textdomain, locale_dir)
-    except ImportError, ioe:
-        pass
 
 del locale_dir
 
@@ -124,19 +124,31 @@ user_hooks = None
 # Episode states used in the database
 STATE_NORMAL, STATE_DOWNLOADED, STATE_DELETED = range(3)
 
+# Paths (gPodder's home folder, config, db and download folder)
+home = None
+config_file = None
+database_file = None
+downloads = None
+
+# Function to set a new gPodder home folder
+def set_home(new_home):
+    global home, config_file, database_file, downloads
+    home = os.path.abspath(new_home)
+
+    config_file = os.path.join(home, 'Settings')
+    database_file = os.path.join(home, 'Database')
+    downloads = os.path.join(home, 'Downloads')
+
 # Default locations for configuration and data files
-default_home = os.path.expanduser(os.path.join('~', '.config', 'gpodder'))
-home = os.environ.get('GPODDER_HOME', None)
-if home is None:
-    home = default_home
-else:
-    print >>sys.stderr, 'Using', home, 'to store data (GPODDER_HOME is set)'
-subscription_file = os.path.join(home, 'channels.opml')
-config_file = os.path.join(home, 'gpodder.conf')
-database_file = os.path.join(home, 'database.sqlite')
+default_home = os.path.expanduser(os.path.join('~', 'gPodder'))
+set_home(os.environ.get('GPODDER_HOME', default_home))
+
+if home != default_home:
+    print >>sys.stderr, 'Storing data in', home, '(GPODDER_HOME is set)'
 
 # Plugins to load by default
-DEFAULT_PLUGINS = ['gpodder.soundcloud', 'gpodder.xspf']
+DEFAULT_PLUGINS = ['gpodder.plugins.soundcloud', 'gpodder.plugins.xspf',
+                   'gpodder.plugins.woodchuck']
 
 def load_plugins():
     """Load (non-essential) plugin modules
@@ -144,7 +156,6 @@ def load_plugins():
     This loads a default set of plugins, but you can use
     the environment variable "GPODDER_PLUGINS" to modify
     the list of plugins."""
-    global DEFAULT_PLUGINS
     PLUGINS = os.environ.get('GPODDER_PLUGINS', None)
     if PLUGINS is None:
         PLUGINS = DEFAULT_PLUGINS
@@ -156,3 +167,26 @@ def load_plugins():
         except Exception, e:
             print >>sys.stderr, 'Cannot load plugin: %s (%s)' % (plugin, e)
 
+
+def detect_platform():
+    global ui
+
+    try:
+        ui.fremantle = ('Maemo 5' in open('/etc/issue').read())
+        if ui.fremantle:
+            print >>sys.stderr, 'Detected platform: Maemo 5 (Fremantle)'
+    except Exception, e:
+        ui.fremantle = False
+
+    try:
+        ui.harmattan = ('MeeGo 1.2 Harmattan' in open('/etc/issue').read())
+    except Exception, e:
+        ui.harmattan = False
+
+    ui.fremantle = ui.fremantle or ui.harmattan
+    ui.desktop = not ui.fremantle and not ui.harmattan
+
+    if ui.fremantle and 'GPODDER_HOME' not in os.environ:
+        new_home = os.path.expanduser(os.path.join('~', 'MyDocs', 'gPodder'))
+        set_home(os.path.expanduser(new_home))
+
