@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 # gPodder - A media aggregator and podcast client
-# Copyright (c) 2005-2012 Thomas Perl and the gPodder Team
+# Copyright (c) 2005-2011 Thomas Perl and the gPodder Team
 #
 # gPodder is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -33,17 +33,21 @@ class gPodderChannel(BuilderWidget):
         self.gPodderChannel.set_title( self.channel.title)
         self.entryTitle.set_text( self.channel.title)
         self.labelURL.set_text(self.channel.url)
-        self.cbSkipFeedUpdate.set_active(not self.channel.feed_update_enabled)
+        self.cbSkipFeedUpdate.set_active(self.channel.pause_subscription)
+
+        self.combo_section.child.set_text(self.channel.section)
+        self.section_list = gtk.ListStore(str)
+        for section in sorted(self.sections):
+            self.section_list.append([section])
+        self.combo_section.set_model(self.section_list)
 
         self.LabelDownloadTo.set_text( self.channel.save_dir)
         self.LabelWebsite.set_text( self.channel.link)
 
-        self.cbNoSync.set_active( not self.channel.sync_to_devices)
-        self.musicPlaylist.set_text(self.channel.device_playlist_name)
-        if self.channel.username:
-            self.FeedUsername.set_text( self.channel.username)
-        if self.channel.password:
-            self.FeedPassword.set_text( self.channel.password)
+        if self.channel.auth_username:
+            self.FeedUsername.set_text( self.channel.auth_username)
+        if self.channel.auth_password:
+            self.FeedPassword.set_text( self.channel.auth_password)
 
         self.cover_downloader.register('cover-available', self.cover_download_finished)
         self.cover_downloader.request_cover(self.channel)
@@ -103,16 +107,24 @@ class gPodderChannel(BuilderWidget):
         self.cover_downloader.unregister('cover-available', self.cover_download_finished)
 
     def on_btnOK_clicked(self, widget, *args):
-        self.channel.sync_to_devices = not self.cbNoSync.get_active()
-        self.channel.feed_update_enabled = not self.cbSkipFeedUpdate.get_active()
-        self.channel.device_playlist_name = self.musicPlaylist.get_text()
-        self.channel.set_custom_title(self.entryTitle.get_text())
-        self.channel.username = self.FeedUsername.get_text().strip()
-        self.channel.password = self.FeedPassword.get_text()
+        self.channel.pause_subscription = self.cbSkipFeedUpdate.get_active()
+        self.channel.rename(self.entryTitle.get_text())
+        self.channel.auth_username = self.FeedUsername.get_text().strip()
+        self.channel.auth_password = self.FeedPassword.get_text()
+
+        new_section = self.combo_section.child.get_text().strip()
+        if self.channel.section != new_section:
+            self.channel.section = new_section
+            section_changed = True
+        else:
+            section_changed = False
+
         self.channel.save()
 
         self.cover_downloader.reload_cover_from_disk(self.channel)
 
         self.gPodderChannel.destroy()
-        self.callback_closed()
+
+        self.update_podcast_list_model(selected=True,
+                sections_changed=section_changed)
 
