@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 # gPodder - A media aggregator and podcast client
-# Copyright (c) 2005-2012 Thomas Perl and the gPodder Team
+# Copyright (c) 2005-2011 Thomas Perl and the gPodder Team
 #
 # gPodder is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
 
 import gtk
 import pango
-from xml.sax import saxutils
+import cgi
 
 import gpodder
 
@@ -27,7 +27,6 @@ _ = gpodder.gettext
 N_ = gpodder.ngettext
 
 from gpodder import util
-from gpodder.liblogger import log
 
 from gpodder.gtkui.interface.common import BuilderWidget
 
@@ -81,22 +80,20 @@ class gPodderEpisodeSelector(BuilderWidget):
                         calculate the size of an episode; set this to
                         None if no total size calculation should be
                         done (in cases where total size is useless)
-                        (default is 'length')
+                        (default is 'file_size')
       - tooltip_attribute: (optional) The name of an attribute of
                            the supplied episode objects that holds
                            the text for the tooltips when hovering
                            over an episode (default is 'description')
                            
     """
-    finger_friendly_widgets = ['btnCancel', 'btnOK', 'btnCheckAll', 'btnCheckNone', 'treeviewEpisodes']
-    
     COLUMN_INDEX = 0
     COLUMN_TOOLTIP = 1
     COLUMN_TOGGLE = 2
     COLUMN_ADDITIONAL = 3
 
     def new( self):
-        self._config.connect_gtk_window(self.gPodderEpisodeSelector, 'episode_selector', True)
+        self._config.connect_gtk_window(self.gPodderEpisodeSelector, '_episode_selector', True)
         if not hasattr( self, 'callback'):
             self.callback = None
 
@@ -113,7 +110,7 @@ class gPodderEpisodeSelector(BuilderWidget):
             self.episodes = []
 
         if not hasattr( self, 'size_attribute'):
-            self.size_attribute = 'length'
+            self.size_attribute = 'file_size'
 
         if not hasattr(self, 'tooltip_attribute'):
             self.tooltip_attribute = 'description'
@@ -133,9 +130,8 @@ class gPodderEpisodeSelector(BuilderWidget):
         if not hasattr( self, 'columns'):
             self.columns = (('title_markup', None, None, _('Episode')),)
 
-        if hasattr( self, 'title'):
-            self.gPodderEpisodeSelector.set_title( self.title)
-            self.labelHeading.set_markup( '<b><big>%s</big></b>' % saxutils.escape( self.title))
+        if hasattr(self, 'title'):
+            self.gPodderEpisodeSelector.set_title(self.title)
 
         if hasattr( self, 'instructions'):
             self.labelInstructions.set_text( self.instructions)
@@ -196,19 +192,16 @@ class gPodderEpisodeSelector(BuilderWidget):
                 try:
                     tooltip = getattr(episode, self.tooltip_attribute)
                 except:
-                    log('Episode object %s does not have tooltip attribute: "%s"', episode, self.tooltip_attribute, sender=self)
                     tooltip = None
             row = [ index, tooltip, self.selected[index] ]
             for name, sort_name, sort_type, caption in self.columns:
                 if not hasattr(episode, name):
-                    log('Warning: Missing attribute "%s"', name, sender=self)
                     row.append(None)
                 else:
                     row.append(getattr( episode, name))
                     
                 if sort_name is not None:
                     if not hasattr(episode, sort_name):
-                        log('Warning: Missing attribute "%s"', sort_name, sender=self)
                         row.append(None)
                     else:
                         row.append(getattr( episode, sort_name))
@@ -224,7 +217,7 @@ class gPodderEpisodeSelector(BuilderWidget):
                 self.treeviewEpisodes.set_property('has-tooltip', True)
                 self.treeviewEpisodes.connect('query-tooltip', self.treeview_episodes_query_tooltip)
             except:
-                log('I cannot set has-tooltip/query-tooltip (need at least PyGTK 2.12)', sender=self)
+                pass
         self.last_tooltip_episode = None
         self.episode_list_can_tooltip = True
 
@@ -315,7 +308,7 @@ class gPodderEpisodeSelector(BuilderWidget):
                     total_size += int(getattr( episode, self.size_attribute))
                     count += 1
                 except:
-                    log( 'Cannot get size for %s', episode.title, sender = self)
+                    pass
 
             text = []
             if count == 0: 
