@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 # gPodder - A media aggregator and podcast client
-# Copyright (c) 2005-2012 Thomas Perl and the gPodder Team
+# Copyright (c) 2005-2011 Thomas Perl and the gPodder Team
 #
 # gPodder is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -37,8 +37,6 @@ from gpodder.gtkui.interface.common import BuilderWidget
 
 
 class gPodderPodcastDirectory(BuilderWidget):
-    finger_friendly_widgets = ['btnDownloadOpml', 'btnCancel', 'btnOK', 'treeviewChannelChooser']
-    
     def new(self):
         if hasattr(self, 'custom_title'):
             self.gPodderPodcastDirectory.set_title(self.custom_title)
@@ -108,7 +106,7 @@ class gPodderPodcastDirectory(BuilderWidget):
 
     def thread_func(self, tab=0):
         if tab == 1:
-            model = OpmlListModel(opml.Importer(self._config.toplist_url))
+            model = OpmlListModel(opml.Importer(self._config.toplist_opml))
             if len(model) == 0:
                 self.notification(_('The specified URL does not provide any valid OPML podcast items.'), _('No feeds found'))
         elif tab == 2:
