@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 # gPodder - A media aggregator and podcast client
-# Copyright (c) 2005-2012 Thomas Perl and the gPodder Team
+# Copyright (c) 2005-2011 Thomas Perl and the gPodder Team
 #
 # gPodder is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -66,57 +66,8 @@ class NewEpisodeActionList(gtk.ListStore):
         self._config.auto_download = self[index][self.C_AUTO_DOWNLOAD]
 
 
-class DeviceTypeActionList(gtk.ListStore):
-    C_CAPTION, C_DEVICE_TYPE = range(2)
-    ACTION_NONE, ACTION_ASK, ACTION_MINIMIZED, ACTION_ALWAYS = range(4)
-
-    def __init__(self, config):
-        gtk.ListStore.__init__(self, str, str)
-        self._config = config
-        self.append((_('None'), 'none'))
-        self.append((_('iPod'), 'ipod'))
-        self.append((_('Filesystem-based'), 'filesystem'))
-        self.append((_('MTP'), 'mtp'))
-
-    def get_index(self):
-        for index, row in enumerate(self):
-            if self._config.device_type == row[self.C_DEVICE_TYPE]:
-                return index
-        return 0 # Some sane default
-
-    def set_index(self, index):
-        self._config.device_type = self[index][self.C_DEVICE_TYPE]
-
-class OnSyncActionList(gtk.ListStore):
-    C_CAPTION, C_ON_SYNC_DELETE, C_ON_SYNC_MARK_PLAYED = range(3)
-    ACTION_NONE, ACTION_ASK, ACTION_MINIMIZED, ACTION_ALWAYS = range(4)
-
-    def __init__(self, config):
-        gtk.ListStore.__init__(self, str, bool, bool)
-        self._config = config
-        self.append((_('Do nothing'), False, False))
-        self.append((_('Mark it as played'), False, True))
-        self.append((_('Delete it from gPodder'), True, False))
-
-    def get_index(self):
-        for index, row in enumerate(self):
-            if self._config.on_sync_delete and row[self.C_ON_SYNC_DELETE]:
-                return index
-            if self._config.on_sync_mark_played and row[self.C_ON_SYNC_MARK_PLAYED] \
-                    and not self._config.on_sync_delete:
-                return index
-        return 0 # Some sane default
-
-    def set_index(self, index):
-        self._config.on_sync_delete = self[index][self.C_ON_SYNC_DELETE]
-        self._config.on_sync_mark_played = self[index][self.C_ON_SYNC_MARK_PLAYED]
-
-
 class gPodderPreferences(BuilderWidget):
     def new(self):
-        if not hasattr(self, 'callback_finished'):
-            self.callback_finished = None
-
         for cb in (self.combo_audio_player_app, self.combo_video_player_app):
             cellrenderer = gtk.CellRendererPixbuf()
             cb.pack_start(cellrenderer, False)
@@ -138,7 +89,6 @@ class gPodderPreferences(BuilderWidget):
         self.combo_video_player_app.set_active(index)
 
         self._config.connect_gtk_togglebutton('enable_notifications', self.checkbutton_enable_notifications)
-        self._config.connect_gtk_togglebutton('display_tray_icon', self.checkbutton_show_tray_icon)
 
         self.update_interval_presets = [0, 10, 30, 60, 2*60, 6*60, 12*60]
         adjustment_update_interval = self.hscale_update_interval.get_adjustment()
@@ -155,7 +105,6 @@ class gPodderPreferences(BuilderWidget):
             index = self.update_interval_presets.index(self._config.auto_update_frequency)
             self.hscale_update_interval.set_value(index)
 
-        self._config.connect_gtk_togglebutton('update_on_startup', self.checkbutton_update_on_startup)
         self._config.connect_gtk_spinbutton('max_episodes_per_feed', self.spinbutton_episode_limit)
 
         self.auto_download_model = NewEpisodeActionList(self._config)
@@ -176,23 +125,7 @@ class gPodderPreferences(BuilderWidget):
             self.hscale_expiration.set_value(0)
 
         self._config.connect_gtk_togglebutton('auto_remove_unplayed_episodes', self.checkbutton_expiration_unplayed)
-        self._config.connect_gtk_togglebutton('auto_cleanup_downloads', self.checkbutton_auto_cleanup_downloads)
-
-        self.device_type_model = DeviceTypeActionList(self._config)
-        self.combobox_device_type.set_model(self.device_type_model)
-        cellrenderer = gtk.CellRendererText()
-        self.combobox_device_type.pack_start(cellrenderer, True)
-        self.combobox_device_type.add_attribute(cellrenderer, 'text', DeviceTypeActionList.C_CAPTION)
-        self.combobox_device_type.set_active(self.device_type_model.get_index())
-
-        self.on_sync_model = OnSyncActionList(self._config)
-        self.combobox_on_sync.set_model(self.on_sync_model)
-        cellrenderer = gtk.CellRendererText()
-        self.combobox_on_sync.pack_start(cellrenderer, True)
-        self.combobox_on_sync.add_attribute(cellrenderer, 'text', OnSyncActionList.C_CAPTION)
-        self.combobox_on_sync.set_active(self.on_sync_model.get_index())
-
-        self._config.connect_gtk_togglebutton('only_sync_not_played', self.checkbutton_only_sync_not_played)
+        self._config.connect_gtk_togglebutton('auto_remove_unfinished_episodes', self.checkbutton_expiration_unfinished)
 
         # Have to do this before calling set_active on checkbutton_enable
         self._enable_mygpo = self._config.mygpo_enabled
@@ -214,9 +147,6 @@ class gPodderPreferences(BuilderWidget):
         # Flush settings for mygpo client now
         self.mygpo_client.flush(now=True)
 
-        if self.callback_finished:
-            self.callback_finished()
-
     def on_button_close_clicked(self, widget):
         self.main_window.destroy()
 
@@ -270,50 +200,6 @@ class gPodderPreferences(BuilderWidget):
         index = self.combo_auto_download.get_active()
         self.auto_download_model.set_index(index)
 
-    def on_combobox_on_sync_changed(self, widget):
-        index = self.combobox_on_sync.get_active()
-        self.on_sync_model.set_index(index)
-
-    def on_combobox_device_type_changed(self, widget):
-        index = self.combobox_device_type.get_active()
-        self.device_type_model.set_index(index)
-        if index == 0:
-            self.btn_filesystemMountpoint.set_label('')
-            self.btn_filesystemMountpoint.set_sensitive(False)
-        if index == 1:
-            self.btn_filesystemMountpoint.set_label(self._config.ipod_mount)
-            self.btn_filesystemMountpoint.set_sensitive(True)
-            self._config.connect_gtk_togglebutton('ipod_purge_old_episodes', self.checkbutton_delete_played)
-        if index == 2:
-            self.btn_filesystemMountpoint.set_label(self._config.mp3_player_folder)
-            self.btn_filesystemMountpoint.set_sensitive(True)
-            self._config.connect_gtk_togglebutton('mp3_player_delete_played', self.checkbutton_delete_played)
-        if index == 3:
-            self.btn_filesystemMountpoint.set_label('')
-            self.btn_filesystemMountpoint.set_sensitive(False)
-
-        children = self.btn_filesystemMountpoint.get_children()
-        if children:
-            label = children.pop()
-            label.set_alignment(0., .5)
-
-    def on_btn_device_mountpoint_clicked(self, widget):
-        fs = gtk.FileChooserDialog( title = _('Select folder for mount point'), action = gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER)
-        fs.add_button( gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)
-        fs.add_button( gtk.STOCK_OPEN, gtk.RESPONSE_OK)
-        fs.set_current_folder(self.btn_filesystemMountpoint.get_label())
-        if fs.run() == gtk.RESPONSE_OK:
-            filename = fs.get_filename()
-            if self._config.device_type == 'filesystem':
-                self._config.mp3_player_folder = filename
-            elif self._config.device_type == 'ipod':
-                self._config.ipod_mount = filename
-
-            # Request an update of the mountpoint button
-            self.on_combobox_device_type_changed(None)
-
-        fs.destroy()
-
     def format_expiration_value(self, scale, value):
         value = int(value)
         if value == 0:
@@ -333,6 +219,7 @@ class gPodderPreferences(BuilderWidget):
             self._config.episode_old_age = value
 
         self.checkbutton_expiration_unplayed.set_sensitive(value > 0)
+        self.checkbutton_expiration_unfinished.set_sensitive(value > 0)
 
     def on_enabled_toggled(self, widget):
         # Only update indirectly (see on_dialog_destroy)
