@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 # gPodder - A media aggregator and podcast client
-# Copyright (c) 2005-2012 Thomas Perl and the gPodder Team
+# Copyright (c) 2005-2011 Thomas Perl and the gPodder Team
 #
 # gPodder is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -22,14 +22,15 @@ import gtk.gdk
 import gobject
 import pango
 import os
+import cgi
 
-from xml.sax import saxutils
 
 import gpodder
 
 _ = gpodder.gettext
 
-from gpodder.liblogger import log
+import logging
+logger = logging.getLogger(__name__)
 
 from gpodder import util
 
@@ -62,7 +63,7 @@ class gPodderShownotes(gPodderShownotesBase):
                     setattr(self, 'have_webkit', True)
                     setattr(self, 'htmlview', webkit.WebView())
                 else:
-                    log('Your WebKit is too old (see bug 1001).', sender=self)
+                    logger.warn('Your WebKit is too old (gPodder bug 1001).')
                     setattr(self, 'have_webkit', False)
 
                 def navigation_policy_decision(wv, fr, req, action, decision):
@@ -126,13 +127,11 @@ class gPodderShownotes(gPodderShownotesBase):
 
             # Get the description - if it looks like plaintext, replace the
             # newline characters with line breaks for the HTML view
-            description = self.episode.description
-            if '<' not in description:
-                description = description.replace('\n', '<br>')
+            description = self.episode.description_html
 
             args = (
-                    saxutils.escape(heading),
-                    saxutils.escape(subheading),
+                    cgi.escape(heading),
+                    cgi.escape(subheading),
                     self.episode.get_play_info_string(),
                     description,
             )
