@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 # gPodder - A media aggregator and podcast client
-# Copyright (c) 2005-2012 Thomas Perl and the gPodder Team
+# Copyright (c) 2005-2011 Thomas Perl and the gPodder Team
 #
 # gPodder is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -52,14 +52,6 @@ class DownloadStatusModel(gtk.ListStore):
         self._status_ids[download.DownloadTask.CANCELLED] = gtk.STOCK_CANCEL
         self._status_ids[download.DownloadTask.PAUSED] = gtk.STOCK_MEDIA_PAUSE
 
-        ICON = lambda x: x
-
-        if gpodder.ui.fremantle:
-            self._status_ids[download.DownloadTask.DOWNLOADING] = ICON('gpodder-download')
-            self._status_ids[download.DownloadTask.FAILED] = ICON('general_stop')
-            self._status_ids[download.DownloadTask.PAUSED] = ICON('camera_video_pause')
-            self._status_ids[download.DownloadTask.DONE] = ICON('general_tickmark_checked')
-
     def _format_message(self, episode, message, podcast):
         return '%s\n<small>%s - %s</small>' % (episode, message, podcast)
 
@@ -143,17 +135,6 @@ class DownloadStatusModel(gtk.ListStore):
 
         return False
 
-    def cancel_by_url(self, url):
-        for row in self:
-            task = row[DownloadStatusModel.C_TASK]
-            if task is not None and task.url == url and \
-                    task.status in (task.DOWNLOADING, \
-                                    task.QUEUED):
-                task.status = task.CANCELLED
-                return True
-
-        return False
-
 
 class DownloadTaskMonitor(object):
     """A helper class that abstracts download events"""
