@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 # gPodder - A media aggregator and podcast client
-# Copyright (c) 2005-2012 Thomas Perl and the gPodder Team
+# Copyright (c) 2005-2011 Thomas Perl and the gPodder Team
 #
 # gPodder is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -29,8 +29,6 @@ from gpodder import util
 
 
 class gPodderAddPodcast(BuilderWidget):
-    finger_friendly_widgets = ['btn_close', 'btn_add' 'btn_paste']
-
     def new(self):
         if not hasattr(self, 'add_urls_callback'):
             self.add_urls_callback = None
@@ -41,17 +39,6 @@ class gPodderAddPodcast(BuilderWidget):
         if hasattr(self, 'preset_url'):
             self.entry_url.set_text(self.preset_url)
         self.entry_url.connect('activate', self.on_entry_url_activate)
-        if gpodder.ui.fremantle:
-            self.btn_add = self.main_window.add_button(gtk.STOCK_ADD, 0)
-            self.btn_add.connect('clicked', self.on_btn_add_clicked)
-            self.on_entry_url_changed(self.entry_url) # for sensitivity
-            self.btn_add.show_all()
-            # Deactivate capitalization and word completion (Maemo bug 5184)
-            self.entry_url.set_property('hildon-input-mode', \
-                    gtk.HILDON_GTK_INPUT_MODE_FULL)
-        elif gpodder.ui.diablo:
-            self.entry_url.set_property('hildon-input-mode', \
-                    'HILDON_GTK_INPUT_MODE_FULL')
         self.gPodderAddPodcast.show()
 
         if not hasattr(self, 'preset_url'):
