@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 # gPodder - A media aggregator and podcast client
-# Copyright (c) 2005-2012 Thomas Perl and the gPodder Team
+# Copyright (c) 2005-2011 Thomas Perl and the gPodder Team
 #
 # gPodder is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -53,19 +53,18 @@ class ProgressIndicator(object):
         return True
 
     def _create_progress(self):
-        if gpodder.ui.fremantle:
-            self.dialog = gtk.Dialog(self.title, self.parent, 0, \
-                    (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL))
-            import hildon
-            hildon.hildon_gtk_window_set_progress_indicator(self.dialog, True)
-        else:
-            self.dialog = gtk.MessageDialog(self.parent, \
-                    0, 0, gtk.BUTTONS_CANCEL, self.subtitle or self.title)
-            self.dialog.label.set_selectable(False)
+        self.dialog = gtk.MessageDialog(self.parent, \
+                0, 0, gtk.BUTTONS_CANCEL, self.subtitle or self.title)
         self.dialog.connect('delete-event', self._on_delete_event)
         self.dialog.set_title(self.title)
         self.dialog.set_deletable(self.cancellable)
 
+        # Avoid selectable text (requires PyGTK >= 2.22)
+        if hasattr(self.dialog, 'get_message_area'):
+            for label in self.dialog.get_message_area():
+                if isinstance(label, gtk.Label):
+                    label.set_selectable(False)
+
         self.dialog.set_response_sensitive(gtk.RESPONSE_CANCEL, \
                 self.cancellable)
 
@@ -80,9 +79,8 @@ class ProgressIndicator(object):
             self.progressbar.set_text(self._initial_message)
 
         self.dialog.vbox.add(self.progressbar)
-        if not gpodder.ui.fremantle:
-            self.indicator = SpinningProgressIndicator()
-            self.dialog.set_image(self.indicator)
+        self.indicator = SpinningProgressIndicator()
+        self.dialog.set_image(self.indicator)
         self.dialog.show_all()
 
         gobject.source_remove(self.source_id)
