@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 # gPodder - A media aggregator and podcast client
-# Copyright (c) 2005-2012 Thomas Perl and the gPodder Team
+# Copyright (c) 2005-2011 Thomas Perl and the gPodder Team
 #
 # gPodder is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -34,7 +34,7 @@ class gPodderShownotesBase(BuilderWidget):
         setattr(self, 'task', None)
 
         self._config.connect_gtk_window(self.main_window, \
-                'episode_window', True)
+                '_episode_window', True)
         self.main_window.connect('delete-event', self._on_delete_event)
         self.main_window.connect('key-press-event', self._on_key_press_event)
         self.on_create_window()
@@ -143,8 +143,6 @@ class gPodderShownotesBase(BuilderWidget):
         """Called from main window for download status changes"""
         if self.main_window.get_property('visible'):
             self.task = task
-            if self.episode is not None:
-                self.episode.reload_from_db()
             self.on_episode_status_changed()
 
     def _download_status_progress(self):
@@ -154,17 +152,11 @@ class gPodderShownotesBase(BuilderWidget):
 
     #############################################################
 
-    def episode_is_new(self):
-        if self.episode is None:
-            return False
-        else:
-            return self.episode.check_is_new(downloading=\
-                    self._episode_is_downloading)
-
-    #############################################################
-
     def show(self, episode):
         if self.main_window.get_property('visible'):
+            if episode == self.episode:
+                return
+
             self.episode = None
             self.task = None
             self.on_hide_window()
