@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 # gPodder - A media aggregator and podcast client
-# Copyright (c) 2005-2012 Thomas Perl and the gPodder Team
+# Copyright (c) 2005-2011 Thomas Perl and the gPodder Team
 #
 # gPodder is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -28,14 +28,18 @@ import gpodder
 _ = gpodder.gettext
 
 from gpodder import util
-from gpodder.liblogger import log
+from gpodder import model
+from gpodder import query
+
+import logging
+logger = logging.getLogger(__name__)
 
 from gpodder.gtkui import draw
 
 import os
 import gtk
-import gobject
-import xml.sax.saxutils
+import cgi
+import re
 
 try:
     import gio
@@ -43,15 +47,69 @@ try:
 except ImportError:
     have_gio = False
 
+# ----------------------------------------------------------
+
+class GEpisode(model.PodcastEpisode):
+    __slots__ = ()
+
+    @property
+    def title_markup(self):
+        return '%s\n<small>%s</small>' % (cgi.escape(self.title),
+                          cgi.escape(self.channel.title))
+
+    @property
+    def markup_new_episodes(self):
+        if self.file_size > 0:
+            length_str = '%s; ' % util.format_filesize(self.file_size)
+        else:
+            length_str = ''
+        return ('<b>%s</b>\n<small>%s'+_('released %s')+ \
+                '; '+_('from %s')+'</small>') % (\
+                cgi.escape(re.sub('\s+', ' ', self.title)), \
+                cgi.escape(length_str), \
+                cgi.escape(self.pubdate_prop), \
+                cgi.escape(re.sub('\s+', ' ', self.channel.title)))
+
+    @property
+    def markup_delete_episodes(self):
+        if self.total_time and self.current_position:
+            played_string = self.get_play_info_string()
+        elif not self.is_new:
+            played_string = _('played')
+        else:
+            played_string = _('unplayed')
+        downloaded_string = self.get_age_string()
+        if not downloaded_string:
+            downloaded_string = _('today')
+        return ('<b>%s</b>\n<small>%s; %s; '+_('downloaded %s')+ \
+                '; '+_('from %s')+'</small>') % (\
+                cgi.escape(self.title), \
+                cgi.escape(util.format_filesize(self.file_size)), \
+                cgi.escape(played_string), \
+                cgi.escape(downloaded_string), \
+                cgi.escape(self.channel.title))
+
+class GPodcast(model.PodcastChannel):
+    __slots__ = ()
+
+    EpisodeClass = GEpisode
+
+class Model(model.Model):
+    PodcastClass = GPodcast
+
+# ----------------------------------------------------------
+
+# Singleton indicator if a row is a section
+class SeparatorMarker(object): pass
+class SectionMarker(object): pass
+
 class EpisodeListModel(gtk.ListStore):
     C_URL, C_TITLE, C_FILESIZE_TEXT, C_EPISODE, C_STATUS_ICON, \
             C_PUBLISHED_TEXT, C_DESCRIPTION, C_TOOLTIP, \
             C_VIEW_SHOW_UNDELETED, C_VIEW_SHOW_DOWNLOADED, \
             C_VIEW_SHOW_UNPLAYED, C_FILESIZE, C_PUBLISHED, \
-            C_TIME, C_TIME_VISIBLE, \
-            C_LOCKED = range(16)
-
-    SEARCH_COLUMNS = (C_TITLE, C_DESCRIPTION)
+            C_TIME, C_TIME_VISIBLE, C_TOTAL_TIME, \
+            C_LOCKED = range(17)
 
     VIEW_ALL, VIEW_UNDELETED, VIEW_DOWNLOADED, VIEW_UNPLAYED = range(4)
 
@@ -61,7 +119,7 @@ class EpisodeListModel(gtk.ListStore):
     def __init__(self, on_filter_changed=lambda has_episodes: None):
         gtk.ListStore.__init__(self, str, str, str, object, \
                 str, str, str, str, bool, bool, bool, \
-                gobject.TYPE_INT64, int, str, bool, bool, bool)
+                int, int, str, bool, int, bool)
 
         # Callback for when the filter / list changes, gets one parameter
         # (has_episodes) that is True if the list has any episodes
@@ -72,48 +130,43 @@ class EpisodeListModel(gtk.ListStore):
         self._sorter = gtk.TreeModelSort(self._filter)
         self._view_mode = self.VIEW_ALL
         self._search_term = None
+        self._search_term_eql = None
         self._filter.set_visible_func(self._filter_visible_func)
 
         # Are we currently showing the "all episodes" view?
         self._all_episodes_view = False
 
-        # "ICON" is used to mark icon names in source files
-        ICON = lambda x: x
-
-        self._icon_cache = {}
-        self.ICON_AUDIO_FILE = ICON('audio-x-generic')
-        self.ICON_VIDEO_FILE = ICON('video-x-generic')
-        self.ICON_IMAGE_FILE = ICON('image-x-generic')
-        self.ICON_GENERIC_FILE = ICON('text-x-generic')
+        self.ICON_AUDIO_FILE = 'audio-x-generic'
+        self.ICON_VIDEO_FILE = 'video-x-generic'
+        self.ICON_IMAGE_FILE = 'image-x-generic'
+        self.ICON_GENERIC_FILE = 'text-x-generic'
         self.ICON_DOWNLOADING = gtk.STOCK_GO_DOWN
         self.ICON_DELETED = gtk.STOCK_DELETE
-        self.ICON_NEW = gtk.STOCK_ABOUT
-        self.ICON_UNPLAYED = ICON('emblem-new')
-        self.ICON_LOCKED = ICON('emblem-readonly')
-        self.ICON_MISSING = ICON('emblem-unreadable')
 
         if 'KDE_FULL_SESSION' in os.environ:
             # Workaround until KDE adds all the freedesktop icons
             # See https://bugs.kde.org/show_bug.cgi?id=233505 and
             #     http://gpodder.org/bug/553
-            self.ICON_DELETED = ICON('archive-remove')
-            self.ICON_UNPLAYED = ICON('vcs-locally-modified')
-            self.ICON_LOCKED = ICON('emblem-locked')
-            self.ICON_MISSING = ICON('vcs-conflicting')
+            self.ICON_DELETED = 'archive-remove'
 
 
     def _format_filesize(self, episode):
-        if episode.length > 0:
-            return util.format_filesize(episode.length, digits=1)
+        if episode.file_size > 0:
+            return util.format_filesize(episode.file_size, 1)
         else:
             return None
 
-
     def _filter_visible_func(self, model, iter):
         # If searching is active, set visibility based on search text
         if self._search_term is not None:
-            key = self._search_term.lower()
-            return any((key in (model.get_value(iter, column) or '').lower()) for column in self.SEARCH_COLUMNS)
+            episode = model.get_value(iter, self.C_EPISODE)
+            if episode is None:
+                return False
+
+            try:
+                return self._search_term_eql.match(episode)
+            except Exception, e:
+                return True
 
         if self._view_mode == self.VIEW_ALL:
             return True
@@ -160,27 +213,27 @@ class EpisodeListModel(gtk.ListStore):
     def set_search_term(self, new_term):
         if self._search_term != new_term:
             self._search_term = new_term
+            self._search_term_eql = query.UserEQL(new_term)
             self._filter.refilter()
             self._on_filter_changed(self.has_episodes())
 
     def get_search_term(self):
         return self._search_term
 
-    def _format_description(self, episode, include_description=False, is_downloading=None):
+    def _format_description(self, episode, include_description=False):
         a, b = '', ''
-        if episode.state != gpodder.STATE_DELETED and not episode.is_played:
+        if episode.state != gpodder.STATE_DELETED and episode.is_new:
             a, b = '<b>', '</b>'
         if include_description and self._all_episodes_view:
-            return '%s%s%s\n<small>%s</small>' % (a, xml.sax.saxutils.escape(episode.title), b,
-                    _('from %s') % xml.sax.saxutils.escape(episode.channel.title))
+            return '%s%s%s\n<small>%s</small>' % (a, cgi.escape(episode.title), b,
+                    _('from %s') % cgi.escape(episode.channel.title))
         elif include_description:
-            return '%s%s%s\n<small>%s</small>' % (a, xml.sax.saxutils.escape(episode.title), b,
-                    xml.sax.saxutils.escape(episode.one_line_description()))
+            return '%s%s%s\n<small>%s</small>' % (a, cgi.escape(episode.title), b,
+                    cgi.escape(episode.one_line_description()))
         else:
-            return ''.join((a, xml.sax.saxutils.escape(episode.title), b))
+            return ''.join((a, cgi.escape(episode.title), b))
 
-    def replace_from_channel(self, channel, downloading=None, \
-            include_description=False, generate_thumbnails=False, \
+    def replace_from_channel(self, channel, include_description=False,
             treeview=None):
         """
         Add episode from the given channel to this model.
@@ -219,62 +272,47 @@ class EpisodeListModel(gtk.ListStore):
                     True, \
                     True, \
                     True, \
-                    episode.length, \
-                    episode.pubDate, \
+                    episode.file_size, \
+                    episode.published, \
                     episode.get_play_info_string(), \
-                    episode.total_time and not episode.current_position, \
-                    episode.total_time and episode.current_position, \
-                    episode.is_locked))
+                    bool(episode.total_time), \
+                    episode.total_time, \
+                    episode.archive))
 
-            self.update_by_iter(iter, downloading, include_description, \
-                    generate_thumbnails, reload_from_db=False)
+            self.update_by_iter(iter, include_description)
 
         self._on_filter_changed(self.has_episodes())
 
-    def update_all(self, downloading=None, include_description=False, \
-            generate_thumbnails=False):
+    def update_all(self, include_description=False):
         for row in self:
-            self.update_by_iter(row.iter, downloading, include_description, \
-                    generate_thumbnails)
+            self.update_by_iter(row.iter, include_description)
 
-    def update_by_urls(self, urls, downloading=None, include_description=False, \
-            generate_thumbnails=False):
+    def update_by_urls(self, urls, include_description=False):
         for row in self:
             if row[self.C_URL] in urls:
-                self.update_by_iter(row.iter, downloading, include_description, \
-                        generate_thumbnails)
+                self.update_by_iter(row.iter, include_description)
 
-    def update_by_filter_iter(self, iter, downloading=None, \
-            include_description=False, generate_thumbnails=False):
+    def update_by_filter_iter(self, iter, include_description=False):
         # Convenience function for use by "outside" methods that use iters
         # from the filtered episode list model (i.e. all UI things normally)
         iter = self._sorter.convert_iter_to_child_iter(None, iter)
-        self.update_by_iter(self._filter.convert_iter_to_child_iter(iter), \
-                downloading, include_description, generate_thumbnails)
+        self.update_by_iter(self._filter.convert_iter_to_child_iter(iter),
+                include_description)
 
-    def update_by_iter(self, iter, downloading=None, include_description=False, \
-            generate_thumbnails=False, reload_from_db=True):
+    def update_by_iter(self, iter, include_description=False):
         episode = self.get_value(iter, self.C_EPISODE)
-        if reload_from_db:
-            episode.reload_from_db()
-
-        if include_description or gpodder.ui.maemo:
-            icon_size = 32
-        else:
-            icon_size = 16
 
         show_bullet = False
         show_padlock = False
         show_missing = False
         status_icon = None
-        status_icon_to_build_from_file = False
         tooltip = []
         view_show_undeleted = True
         view_show_downloaded = False
         view_show_unplayed = False
         icon_theme = gtk.icon_theme_get_default()
 
-        if downloading is not None and downloading(episode):
+        if episode.downloading:
             tooltip.append(_('Downloading'))
             status_icon = self.ICON_DOWNLOADING
             view_show_downloaded = True
@@ -285,17 +323,16 @@ class EpisodeListModel(gtk.ListStore):
                 status_icon = self.ICON_DELETED
                 view_show_undeleted = False
             elif episode.state == gpodder.STATE_NORMAL and \
-                    not episode.is_played:
+                    episode.is_new:
                 tooltip.append(_('New episode'))
-                status_icon = self.ICON_NEW
                 view_show_downloaded = True
                 view_show_unplayed = True
             elif episode.state == gpodder.STATE_DOWNLOADED:
                 tooltip = []
                 view_show_downloaded = True
-                view_show_unplayed = not episode.is_played
-                show_bullet = not episode.is_played
-                show_padlock = episode.is_locked
+                view_show_unplayed = episode.is_new
+                show_bullet = episode.is_new
+                show_padlock = episode.archive
                 show_missing = not episode.file_exists()
                 filename = episode.local_filename(create=False, check_only=True)
 
@@ -309,14 +346,6 @@ class EpisodeListModel(gtk.ListStore):
                 elif file_type == 'image':
                     tooltip.append(_('Downloaded image'))
                     status_icon = self.ICON_IMAGE_FILE
-
-                    # Optional thumbnailing for image downloads
-                    if generate_thumbnails:
-                        if filename is not None:
-                            # set the status icon to the path itself (that
-                            # should be a good identifier anyway)
-                            status_icon = filename
-                            status_icon_to_build_from_file = True
                 else:
                     tooltip.append(_('Downloaded file'))
                     status_icon = self.ICON_GENERIC_FILE
@@ -362,7 +391,7 @@ class EpisodeListModel(gtk.ListStore):
 
         tooltip = ', '.join(tooltip)
 
-        description = self._format_description(episode, include_description, downloading)
+        description = self._format_description(episode, include_description)
         self.set(iter, \
                 self.C_STATUS_ICON, status_icon, \
                 self.C_VIEW_SHOW_UNDELETED, view_show_undeleted, \
@@ -370,119 +399,12 @@ class EpisodeListModel(gtk.ListStore):
                 self.C_VIEW_SHOW_UNPLAYED, view_show_unplayed, \
                 self.C_DESCRIPTION, description, \
                 self.C_TOOLTIP, tooltip, \
-                self.C_TIME, episode.get_play_info_string(), \
-                self.C_TIME_VISIBLE, episode.total_time, \
-                self.C_LOCKED, episode.is_locked)
-
-    def _get_icon_from_image(self,image_path, icon_size):
-        """
-        Load an local image file and transform it into an icon.
-
-        Return a pixbuf scaled to the desired size and may return None
-        if the icon creation is impossible (file not found etc).
-        """
-        if not os.path.exists(image_path):
-            return None
-        # load image from disc (code adapted from CoverDownloader
-        # except that no download is needed here)
-        loader = gtk.gdk.PixbufLoader()
-        pixbuf = None
-        try:
-            loader.write(open(image_path, 'rb').read())
-            loader.close()
-            pixbuf = loader.get_pixbuf()
-        except:
-            log('Data error while loading image %s', image_path, sender=self)
-            return None
-        # Now scale the image with ratio (copied from _resize_pixbuf_keep_ratio)
-        # Resize if too wide
-        if pixbuf.get_width() > icon_size:
-            f = float(icon_size)/pixbuf.get_width()
-            (width, height) = (int(pixbuf.get_width()*f), int(pixbuf.get_height()*f))
-            pixbuf = pixbuf.scale_simple(width, height, gtk.gdk.INTERP_BILINEAR)
-        # Resize if too high
-        if pixbuf.get_height() > icon_size:
-            f = float(icon_size)/pixbuf.get_height()
-            (width, height) = (int(pixbuf.get_width()*f), int(pixbuf.get_height()*f))
-            pixbuf = pixbuf.scale_simple(width, height, gtk.gdk.INTERP_BILINEAR)
-        return pixbuf
-        
-        
-    def _get_tree_icon(self, icon_name, add_bullet=False, \
-            add_padlock=False, add_missing=False, icon_size=32, \
-            build_icon_from_file = False):
-        """
-        Loads an icon from the current icon theme at the specified
-        size, suitable for display in a gtk.TreeView. Additional
-        emblems can be added on top of the icon.
-
-        Caching is used to speed up the icon lookup.
-        
-        The `build_icon_from_file` argument indicates (when True) that
-        the icon has to be created on the fly from a given image
-        file. The `icon_name` argument is then interpreted as the path
-        to this file. Those specific icons will *not be cached*.
-        """
-        
-        # Add all variables that modify the appearance of the icon, so
-        # our cache does not return the same icons for different requests
-        cache_id = (icon_name, add_bullet, add_padlock, add_missing, icon_size)
-
-        if cache_id in self._icon_cache:
-            return self._icon_cache[cache_id]
-
-        icon_theme = gtk.icon_theme_get_default()
-
-        try:
-            if build_icon_from_file:
-                icon = self._get_icon_from_image(icon_name,icon_size)
-            else:
-                icon = icon_theme.load_icon(icon_name, icon_size, 0)
-        except:
-            try:
-                log('Missing icon in theme: %s', icon_name, sender=self)
-                icon = icon_theme.load_icon(gtk.STOCK_DIALOG_QUESTION, \
-                        icon_size, 0)
-            except:
-                log('Please install the GNOME icon theme.', sender=self)
-                icon = gtk.gdk.Pixbuf(gtk.gdk.COLORSPACE_RGB, \
-                        True, 8, icon_size, icon_size)
-
-        if icon and (add_bullet or add_padlock or add_missing):
-            # We'll modify the icon, so use .copy()
-            if add_missing:
-                try:
-                    icon = icon.copy()
-                    # Desaturate the icon so it looks even more "missing"
-                    icon.saturate_and_pixelate(icon, 0.0, False)
-                    emblem = icon_theme.load_icon(self.ICON_MISSING, icon_size/2, 0)
-                    (width, height) = (emblem.get_width(), emblem.get_height())
-                    xpos = icon.get_width() - width
-                    ypos = icon.get_height() - height
-                    emblem.composite(icon, xpos, ypos, width, height, xpos, ypos, 1, 1, gtk.gdk.INTERP_BILINEAR, 255)
-                except:
-                    pass
-            elif add_bullet:
-                try:
-                    icon = icon.copy()
-                    emblem = icon_theme.load_icon(self.ICON_UNPLAYED, icon_size/2, 0)
-                    (width, height) = (emblem.get_width(), emblem.get_height())
-                    xpos = icon.get_width() - width
-                    ypos = icon.get_height() - height
-                    emblem.composite(icon, xpos, ypos, width, height, xpos, ypos, 1, 1, gtk.gdk.INTERP_BILINEAR, 255)
-                except:
-                    pass
-            if add_padlock:
-                try:
-                    icon = icon.copy()
-                    emblem = icon_theme.load_icon(self.ICON_LOCKED, icon_size/2, 0)
-                    (width, height) = (emblem.get_width(), emblem.get_height())
-                    emblem.composite(icon, 0, 0, width, height, 0, 0, 1, 1, gtk.gdk.INTERP_BILINEAR, 255)
-                except:
-                    pass
-
-        self._icon_cache[cache_id] = icon
-        return icon
+                self.C_TIME, episode.get_play_info_string(duration_only=True), \
+                self.C_TIME_VISIBLE, bool(episode.total_time), \
+                self.C_TOTAL_TIME, episode.total_time, \
+                self.C_LOCKED, episode.archive, \
+                self.C_FILESIZE_TEXT, self._format_filesize(episode), \
+                self.C_FILESIZE, episode.file_size)
 
 
 class PodcastChannelProxy(object):
@@ -494,37 +416,22 @@ class PodcastChannelProxy(object):
         self.channels = channels
         self.title =  _('All episodes')
         self.description = _('from all podcasts')
-        self.parse_error = ''
+        #self.parse_error = ''
         self.url = ''
+        self.section = ''
         self.id = None
-        self._save_dir_size_set = False
-        self.save_dir_size = 0L
         self.cover_file = os.path.join(gpodder.images_folder, 'podcast-all.png')
-        self.feed_update_enabled = True
-        self.channel_is_locked = False
-
-    def __getattribute__(self, name):
-        try:
-            return object.__getattribute__(self, name)
-        except AttributeError:
-            log('Unsupported method call (%s)', name, sender=self)
+        self.pause_subscription = False
+        self.auto_archive_episodes = False
 
     def get_statistics(self):
         # Get the total statistics for all channels from the database
-        return self._db.get_total_count()
+        return self._db.get_podcast_statistics()
 
     def get_all_episodes(self):
         """Returns a generator that yields every episode"""
-        channel_lookup_map = dict((c.id, c) for c in self.channels)
-        return self._db.load_all_episodes(channel_lookup_map)
-
-    def request_save_dir_size(self):
-        if not self._save_dir_size_set:
-            self.update_save_dir_size()
-        self._save_dir_size_set = True
-
-    def update_save_dir_size(self):
-        self.save_dir_size = util.calculate_size(self._config.download_dir)
+        return Model.sort_episodes_by_pubdate((e for c in self.channels
+                for e in c.get_all_episodes()), True)
 
 
 class PodcastListModel(gtk.ListStore):
@@ -532,9 +439,9 @@ class PodcastListModel(gtk.ListStore):
             C_COVER, C_ERROR, C_PILL_VISIBLE, \
             C_VIEW_SHOW_UNDELETED, C_VIEW_SHOW_DOWNLOADED, \
             C_VIEW_SHOW_UNPLAYED, C_HAS_EPISODES, C_SEPARATOR, \
-            C_DOWNLOADS = range(14)
+            C_DOWNLOADS, C_COVER_VISIBLE, C_SECTION = range(16)
 
-    SEARCH_COLUMNS = (C_TITLE, C_DESCRIPTION)
+    SEARCH_COLUMNS = (C_TITLE, C_DESCRIPTION, C_SECTION)
 
     @classmethod
     def row_separator_func(cls, model, iter):
@@ -543,7 +450,7 @@ class PodcastListModel(gtk.ListStore):
     def __init__(self, cover_downloader):
         gtk.ListStore.__init__(self, str, str, str, gtk.gdk.Pixbuf, \
                 object, gtk.gdk.Pixbuf, str, bool, bool, bool, bool, \
-                bool, bool, int)
+                bool, bool, int, bool, str)
 
         # Filter to allow hiding some episodes
         self._filter = self.filter_new()
@@ -552,28 +459,23 @@ class PodcastListModel(gtk.ListStore):
         self._filter.set_visible_func(self._filter_visible_func)
 
         self._cover_cache = {}
-        if gpodder.ui.fremantle:
-            self._max_image_side = 64
-        else:
-            self._max_image_side = 40
+        self._max_image_side = 40
         self._cover_downloader = cover_downloader
 
-        # "ICON" is used to mark icon names in source files
-        ICON = lambda x: x
-
-        #self.ICON_DISABLED = ICON('emblem-unreadable')
-        self.ICON_DISABLED = ICON('gtk-media-pause')
+        self.ICON_DISABLED = 'gtk-media-pause'
 
     def _filter_visible_func(self, model, iter):
         # If searching is active, set visibility based on search text
         if self._search_term is not None:
+            if model.get_value(iter, self.C_CHANNEL) == SectionMarker:
+                return True
             key = self._search_term.lower()
             columns = (model.get_value(iter, c) for c in self.SEARCH_COLUMNS)
             return any((key in c.lower() for c in columns if c is not None))
 
         if model.get_value(iter, self.C_SEPARATOR):
             return True
-        if self._view_mode == EpisodeListModel.VIEW_ALL:
+        elif self._view_mode == EpisodeListModel.VIEW_ALL:
             return model.get_value(iter, self.C_HAS_EPISODES)
         elif self._view_mode == EpisodeListModel.VIEW_UNDELETED:
             return model.get_value(iter, self.C_VIEW_SHOW_UNDELETED)
@@ -683,7 +585,7 @@ class PodcastListModel(gtk.ListStore):
 
         pixbuf = self._cover_downloader.get_cover(channel, avoid_downloading=True)
         pixbuf_overlay = self._resize_pixbuf(channel.url, pixbuf)
-        if add_overlay and not channel.feed_update_enabled:
+        if add_overlay and channel.pause_subscription:
             pixbuf_overlay = self._overlay_pixbuf(pixbuf_overlay, self.ICON_DISABLED)
             pixbuf_overlay.saturate_and_pixelate(pixbuf_overlay, 0.0, False)
 
@@ -697,33 +599,38 @@ class PodcastListModel(gtk.ListStore):
 
     def _format_description(self, channel, total, deleted, \
             new, downloaded, unplayed):
-        title_markup = xml.sax.saxutils.escape(channel.title)
-        if channel.feed_update_enabled:
-            description_markup = xml.sax.saxutils.escape(util.get_first_line(channel.description) or ' ')
+        title_markup = cgi.escape(channel.title)
+        if not channel.pause_subscription:
+            description_markup = cgi.escape(util.get_first_line(channel.description) or ' ')
         else:
-            description_markup = xml.sax.saxutils.escape(_('Subscription paused'))
+            description_markup = cgi.escape(_('Subscription paused'))
         d = []
         if new:
             d.append('<span weight="bold">')
         d.append(title_markup)
         if new:
             d.append('</span>')
-        return ''.join(d+['\n', '<small>', description_markup, '</small>'])
+
+        if description_markup.strip():
+            return ''.join(d+['\n', '<small>', description_markup, '</small>'])
+        else:
+            return ''.join(d)
 
     def _format_error(self, channel):
-        if channel.parse_error:
-            return str(channel.parse_error)
-        else:
-            return None
+        #if channel.parse_error:
+        #    return str(channel.parse_error)
+        #else:
+        #    return None
+        return None
 
     def set_channels(self, db, config, channels):
         # Clear the model and update the list of podcasts
         self.clear()
 
         def channel_to_row(channel, add_overlay=False):
-            return (channel.url, '', '', None, channel, \
-                    self._get_cover_image(channel, add_overlay), '', True, True, True, \
-                    True, True, False, 0)
+            return (channel.url, '', '', None, channel,
+                    self._get_cover_image(channel, add_overlay), '', True,
+                    True, True, True, True, False, 0, True, '')
 
         if config.podcast_list_view_all and channels:
             all_episodes = PodcastChannelProxy(db, config, channels)
@@ -731,13 +638,40 @@ class PodcastListModel(gtk.ListStore):
             self.update_by_iter(iter)
 
             # Separator item
-            self.append(('', '', '', None, None, None, '', True, True, \
-                    True, True, True, True, 0))
+            if not config.podcast_list_sections:
+                self.append(('', '', '', None, SeparatorMarker, None, '',
+                    True, True, True, True, True, True, 0, False, ''))
+
+        def key_func(pair):
+            section, podcast = pair
+            return (section, model.Model.podcast_sort_key(podcast))
+
+        if config.podcast_list_sections:
+            def convert(channels):
+                for channel in channels:
+                    yield (channel.group_by, channel)
+        else:
+            def convert(channels):
+                for channel in channels:
+                    yield (None, channel)
+
+        added_sections = []
+        old_section = None
+        for section, channel in sorted(convert(channels), key=key_func):
+            if old_section != section:
+                it = self.append(('-', section, '', None, SectionMarker, None,
+                    '', True, True, True, True, True, False, 0, False, section))
+                added_sections.append(it)
+                old_section = section
 
-        for channel in channels:
             iter = self.append(channel_to_row(channel, True))
             self.update_by_iter(iter)
 
+        # Update section header stats only after all podcasts
+        # have been added to the list to get the stats right
+        for it in added_sections:
+            self.update_by_iter(it)
+
     def get_filter_path_from_url(self, url):
         # Return the path of the filtered model for a given URL
         child_path = self.get_path_from_url(url)
@@ -778,27 +712,55 @@ class PodcastListModel(gtk.ListStore):
         for row in self:
             self.update_by_iter(row.iter)
 
+    def update_sections(self):
+        for row in self:
+            if row[self.C_CHANNEL] is SectionMarker:
+                self.update_by_iter(row.iter)
+
     def update_by_iter(self, iter):
+        if iter is None:
+            return
+
         # Given a GtkTreeIter, update volatile information
-        try:
-            channel = self.get_value(iter, self.C_CHANNEL)
-        except TypeError, te:
-            return
-        if channel is None:
+        channel = self.get_value(iter, self.C_CHANNEL)
+
+        if channel is SectionMarker:
+            section = self.get_value(iter, self.C_TITLE)
+
+            # This row is a section header - update its visibility flags
+            channels = [c for c in (row[self.C_CHANNEL] for row in self)
+                    if isinstance(c, GPodcast) and c.section == section]
+
+            # Calculate the stats over all podcasts of this section
+            total, deleted, new, downloaded, unplayed = map(sum,
+                    zip(*[c.get_statistics() for c in channels]))
+
+            # We could customized the section header here with the list
+            # of channels and their stats (i.e. add some "new" indicator)
+            description = '<span size="16000"> </span><b>%s</b>' % (
+                    cgi.escape(section))
+
+            self.set(iter,
+                self.C_DESCRIPTION, description,
+                self.C_SECTION, section,
+                self.C_VIEW_SHOW_UNDELETED, total - deleted > 0,
+                self.C_VIEW_SHOW_DOWNLOADED, downloaded + new > 0,
+                self.C_VIEW_SHOW_UNPLAYED, unplayed + new > 0)
+
+        if (not isinstance(channel, GPodcast) and
+            not isinstance(channel, PodcastChannelProxy)):
             return
+
         total, deleted, new, downloaded, unplayed = channel.get_statistics()
         description = self._format_description(channel, total, deleted, new, \
                 downloaded, unplayed)
 
-        if gpodder.ui.fremantle:
-            # We don't display the pill, so don't generate it
-            pill_image = None
-        else:
-            pill_image = self._get_pill_image(channel, downloaded, unplayed)
+        pill_image = self._get_pill_image(channel, downloaded, unplayed)
 
         self.set(iter, \
                 self.C_TITLE, channel.title, \
                 self.C_DESCRIPTION, description, \
+                self.C_SECTION, channel.section, \
                 self.C_ERROR, self._format_error(channel), \
                 self.C_PILL, pill_image, \
                 self.C_PILL_VISIBLE, pill_image != None, \
@@ -811,7 +773,7 @@ class PodcastListModel(gtk.ListStore):
     def add_cover_by_channel(self, channel, pixbuf):
         # Resize and add the new cover image
         pixbuf = self._resize_pixbuf(channel.url, pixbuf)
-        if not channel.feed_update_enabled:
+        if channel.pause_subscription:
             pixbuf = self._overlay_pixbuf(pixbuf, self.ICON_DISABLED)
             pixbuf.saturate_and_pixelate(pixbuf, 0.0, False)
 
