@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 # gPodder - A media aggregator and podcast client
-# Copyright (c) 2005-2012 Thomas Perl and the gPodder Team
+# Copyright (c) 2005-2011 Thomas Perl and the gPodder Team
 #
 # gPodder is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -27,7 +27,9 @@ import gpodder
 _ = gpodder.gettext
 
 from gpodder.services import ObservableService
-from gpodder.liblogger import log
+
+import logging
+logger = logging.getLogger(__name__)
 
 from gpodder import util
 from gpodder import youtube
@@ -37,33 +39,6 @@ import os
 import urlparse
 import threading
 
-class DependencyModel(gtk.ListStore):
-    C_NAME, C_DESCRIPTION, C_AVAILABLE_TEXT, C_AVAILABLE, C_MISSING = range(5)
-
-    def __init__(self, depman):
-        gtk.ListStore.__init__(self, str, str, str, bool, str)
-
-        for feature_name, description, modules, tools in depman.dependencies:
-            modules_available, module_info = depman.modules_available(modules)
-            tools_available, tool_info = depman.tools_available(tools)
-
-            available = modules_available and tools_available
-            if available:
-                available_str = _('Available')
-            else:
-                available_str = _('Missing dependencies')
-
-            missing_str = []
-            for module in modules:
-                if not module_info[module]:
-                    missing_str.append(_('Python module "%s" not installed') % module)
-            for tool in tools:
-                if not tool_info[tool]:
-                    missing_str.append(_('Command "%s" not installed') % tool)
-            missing_str = '\n'.join(missing_str)
-
-            self.append((feature_name, description, available_str, available, missing_str))
-
 
 class CoverDownloader(ObservableService):
     """
@@ -100,7 +75,7 @@ class CoverDownloader(ObservableService):
         already-downloaded covers and return None
         when we have no cover on the local disk.
         """
-        log('cover download request for %s', channel.url, sender=self)
+        logger.debug('cover download request for %s', channel.url)
         args = [channel, custom_url, True, avoid_downloading]
         threading.Thread(target=self.__get_cover, args=args).start()
 
@@ -156,36 +131,47 @@ class CoverDownloader(ObservableService):
 
         if not os.path.exists(channel.cover_file):
             if url is None:
-                url = channel.image
+                url = channel.cover_url
 
             new_url = youtube.get_real_cover(channel.url)
             if new_url is not None:
                 url = new_url
 
+            if url is None and channel.link is not None and \
+                    channel.link.startswith('http://'):
+                # Try to use the favicon of the linked website's host
+                split_result = urlparse.urlsplit(channel.link)
+                scheme, netloc, path, query, fragment = split_result
+                path = '/favicon.ico'
+                query = ''
+                split_result = (scheme, netloc, path, query, fragment)
+                url = urlparse.urlunsplit(split_result)
+                logger.debug('Trying favicon: %s', url)
+
             if url is not None:
                 image_data = None
                 try:
-                    log('Trying to download: %s', url, sender=self)
+                    logger.debug('Trying to download: %s', url)
 
                     image_data = util.urlopen(url).read()
                 except:
-                    log('Cannot get image from %s', url, sender=self)
+                    logger.warn('Cannot get image from %s', url, exc_info=True)
 
                 if image_data is not None:
-                    log('Saving image data to %s', channel.cover_file, sender=self)
+                    logger.debug('Saving image data to %s', channel.cover_file)
                     try:
                         fp = open(channel.cover_file, 'wb')
                         fp.write(image_data)
                         fp.close()
                     except IOError, ioe:
-                        log('Cannot save image due to I/O error', sender=self, traceback=True)
+                        logger.error('Cannot save image due to I/O error', exc_info=True)
 
         pixbuf = None
         if os.path.exists(channel.cover_file):
             try:
                 pixbuf = gtk.gdk.pixbuf_new_from_file(channel.cover_file.decode(util.encoding, 'ignore'))
             except:
-                log('Data error while loading %s', channel.cover_file, sender=self)
+                logger.error('Data error while loading %s', channel.cover_file)
 
         if pixbuf is None:
             pixbuf = self.get_default_cover(channel)
