@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 #
 # gPodder - A media aggregator and podcast client
-# Copyright (c) 2005-2012 Thomas Perl and the gPodder Team
+# Copyright (c) 2005-2011 Thomas Perl and the gPodder Team
 #
 # gPodder is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -35,7 +35,8 @@ import sys
 import threading
 import time
 
-from gpodder.liblogger import log
+import logging
+logger = logging.getLogger(__name__)
 
 from gpodder import util
 from gpodder import minidb
@@ -165,7 +166,7 @@ class Change(object):
 
 
 class MygPoClient(object):
-    STORE_FILE = 'mygpo.queue.sqlite'
+    STORE_FILE = 'gpodder.net'
     FLUSH_TIMEOUT = 60
     FLUSH_RETRIES = 3
 
@@ -210,16 +211,60 @@ class MygPoClient(object):
         self._store.remove(rewritten_urls)
         return rewritten_urls
 
-    def get_episode_actions(self, updated_urls):
-        for podcast_url in updated_urls:
-            for action in self._store.load(ReceivedEpisodeAction, \
-                    podcast_url=podcast_url):
-                yield action
+    def process_episode_actions(self, find_episode, on_updated=None):
+        """Process received episode actions
 
-            # Remove all episode actions belonging to this URL
-            self._store.delete(ReceivedEpisodeAction, \
-                    podcast_url=podcast_url)
-            self._store.commit()
+        The parameter "find_episode" should be a function accepting
+        two parameters (podcast_url and episode_url). It will be used
+        to get an episode object that needs to be updated. It should
+        return None if the requested episode does not exist.
+
+        The optional callback "on_updated" should accept a single
+        parameter (the episode object) and will be called whenever
+        the episode data is changed in some way.
+        """
+        logger.debug('Processing received episode actions...')
+        for action in self._store.load(ReceivedEpisodeAction):
+            if action.action not in ('play', 'delete'):
+                # Ignore all other action types for now
+                continue
+
+            episode = find_episode(action.podcast_url, action.episode_url)
+
+            if episode is None:
+                # The episode does not exist on this client
+                continue
+
+            if action.action == 'play':
+                logger.debug('Play action for %s', episode.url)
+                episode.mark(is_played=True)
+
+                if (action.timestamp > episode.current_position_updated and
+                        action.position is not None):
+                    logger.debug('Updating position for %s', episode.url)
+                    episode.current_position = action.position
+                    episode.current_position_updated = action.timestamp
+
+                if action.total:
+                    logger.debug('Updating total time for %s', episode.url)
+                    episode.total_time = action.total
+
+                episode.save()
+                if on_updated is not None:
+                    on_updated(episode)
+            elif action.action == 'delete':
+                if not episode.was_downloaded(and_exists=True):
+                    # Set the episode to a "deleted" state
+                    logger.debug('Marking as deleted: %s', episode.url)
+                    episode.delete_from_disk()
+                    episode.save()
+                    if on_updated is not None:
+                        on_updated(episode)
+
+        # Remove all received episode actions
+        self._store.delete(ReceivedEpisodeAction)
+        self._store.commit()
+        logger.debug('Received episode actions processed.')
 
     def get_received_actions(self):
         """Returns a list of ReceivedSubscribeAction objects
@@ -272,9 +317,9 @@ class MygPoClient(object):
 
     def set_subscriptions(self, urls):
         if self.can_access_webservice():
-            log('Uploading (overwriting) subscriptions...')
+            logger.debug('Uploading (overwriting) subscriptions...')
             self._client.put_subscriptions(self.device_id, urls)
-            log('Subscription upload done.')
+            logger.debug('Subscription upload done.')
         else:
             raise Exception('Webservice access not enabled')
 
@@ -289,19 +334,19 @@ class MygPoClient(object):
                 int(time.time()), None, None, None)
 
     def on_delete(self, episodes):
-        log('Storing %d episode delete actions', len(episodes), sender=self)
+        logger.debug('Storing %d episode delete actions', len(episodes))
         self._store.save(self._convert_episode(e, 'delete') for e in episodes)
 
     def on_download(self, episodes):
-        log('Storing %d episode download actions', len(episodes), sender=self)
+        logger.debug('Storing %d episode download actions', len(episodes))
         self._store.save(self._convert_episode(e, 'download') for e in episodes)
 
     def on_playback_full(self, episode, start, end, total):
-        log('Storing full episode playback action', sender=self)
+        logger.debug('Storing full episode playback action')
         self._store.save(self._convert_played_episode(episode, start, end, total))
 
     def on_playback(self, episodes):
-        log('Storing %d episode playback actions', len(episodes), sender=self)
+        logger.debug('Storing %d episode playback actions', len(episodes))
         self._store.save(self._convert_episode(e, 'play') for e in episodes)
 
     def on_subscribe(self, urls):
@@ -332,7 +377,7 @@ class MygPoClient(object):
             # Store the current contents of the queue database
             self._store.commit()
 
-            log('Worker thread waiting for timeout', sender=self)
+            logger.debug('Worker thread waiting for timeout')
             time.sleep(self.FLUSH_TIMEOUT)
 
         # Only work when enabled, UID set and allowed to work
@@ -340,12 +385,12 @@ class MygPoClient(object):
                 (self._worker_thread is not None or forced):
             self._worker_thread = None
 
-            log('Worker thread starting to work...', sender=self)
+            logger.debug('Worker thread starting to work...')
             for retry in range(self.FLUSH_RETRIES):
                 must_retry = False
 
                 if retry:
-                    log('Retrying flush queue...', sender=self)
+                    logger.debug('Retrying flush queue...')
 
                 # Update the device first, so it can be created if new
                 for action in self._store.load(UpdateDeviceAction):
@@ -372,56 +417,56 @@ class MygPoClient(object):
                     # No more pending actions. Ready to quit.
                     break
 
-            log('Worker thread finished.', sender=self)
+            logger.debug('Worker thread finished.')
         else:
-            log('Worker thread may not execute (disabled).', sender=self)
+            logger.info('Worker thread may not execute (disabled).')
 
         # Store the current contents of the queue database
         self._store.commit()
 
     def flush(self, now=False):
         if not self.can_access_webservice():
-            log('Flush requested, but sync disabled.', sender=self)
+            logger.warn('Flush requested, but sync disabled.')
             return
 
         if self._worker_thread is None or now:
             if now:
-                log('Flushing NOW.', sender=self)
+                logger.debug('Flushing NOW.')
             else:
-                log('Flush requested.', sender=self)
+                logger.debug('Flush requested.')
             self._worker_thread = threading.Thread(target=self._worker_proc, args=[now])
             self._worker_thread.setDaemon(True)
             self._worker_thread.start()
         else:
-            log('Flush requested, already waiting.', sender=self)
+            logger.debug('Flush requested, already waiting.')
 
     def on_config_changed(self, name=None, old_value=None, new_value=None):
         if name in ('mygpo_username', 'mygpo_password', 'mygpo_server') \
                 or self._client is None:
             self._client = api.MygPodderClient(self._config.mygpo_username,
                     self._config.mygpo_password, self._config.mygpo_server)
-            log('Reloading settings.', sender=self)
+            logger.info('Reloading settings.')
         elif name.startswith('mygpo_device_'):
             # Update or create the device
             self.create_device()
 
     def synchronize_episodes(self, actions):
-        log('Starting episode status sync.', sender=self)
+        logger.debug('Starting episode status sync.')
 
         def convert_to_api(action):
             dt = datetime.datetime.utcfromtimestamp(action.timestamp)
-            since = mygpoutil.datetime_to_iso8601(dt)
+            action_ts = mygpoutil.datetime_to_iso8601(dt)
             return api.EpisodeAction(action.podcast_url, \
                     action.episode_url, action.action, \
-                    action.device_id, since, \
+                    action.device_id, action_ts, \
                     action.started, action.position, action.total)
 
         def convert_from_api(action):
             dt = mygpoutil.iso8601_to_datetime(action.timestamp)
-            since = calendar.timegm(dt.timetuple())
+            action_ts = calendar.timegm(dt.timetuple())
             return ReceivedEpisodeAction(action.podcast, \
                     action.episode, action.device, \
-                    action.action, since, \
+                    action.action, action_ts, \
                     action.started, action.position, action.total)
 
         try:
@@ -439,14 +484,13 @@ class MygPoClient(object):
                 changes = self._client.download_episode_actions(since_o.since)
 
                 received_actions = [convert_from_api(a) for a in changes.actions]
-                log('Received %d episode actions', len(received_actions), \
-                        sender=self)
+                logger.debug('Received %d episode actions', len(received_actions))
                 self._store.save(received_actions)
 
                 # Save the "since" value for later use
                 self._store.update(since_o, since=changes.since)
             except Exception, e:
-                log('Exception while polling for episodes.', sender=self, traceback=True)
+                logger.warn('Exception while polling for episodes.', exc_info=True)
 
             # Step 2: Upload Episode actions
 
@@ -458,14 +502,14 @@ class MygPoClient(object):
 
             # Actions have been uploaded to the server - remove them
             self._store.remove(actions)
-            log('Episode actions have been uploaded to the server.', sender=self)
+            logger.debug('Episode actions have been uploaded to the server.')
             return True
         except Exception, e:
-            log('Cannot upload episode actions: %s', str(e), sender=self, traceback=True)
+            logger.error('Cannot upload episode actions: %s', str(e), exc_info=True)
             return False
 
     def synchronize_subscriptions(self, actions):
-        log('Starting subscription sync.', sender=self)
+        logger.debug('Starting subscription sync.')
         try:
             # Load the "since" value from the database
             since_o = self._store.get(SinceValue, host=self.host, \
@@ -485,12 +529,12 @@ class MygPoClient(object):
             # Store received actions for later retrieval (and in case we
             # have outdated actions in the database, simply remove them)
             for url in result.add:
-                log('Received add action: %s', url, sender=self)
+                logger.debug('Received add action: %s', url)
                 self._store.remove(ReceivedSubscribeAction.remove(url))
                 self._store.remove(ReceivedSubscribeAction.add(url))
                 self._store.save(ReceivedSubscribeAction.add(url))
             for url in result.remove:
-                log('Received remove action: %s', url, sender=self)
+                logger.debug('Received remove action: %s', url)
                 self._store.remove(ReceivedSubscribeAction.add(url))
                 self._store.remove(ReceivedSubscribeAction.remove(url))
                 self._store.save(ReceivedSubscribeAction.remove(url))
@@ -502,7 +546,7 @@ class MygPoClient(object):
             remove = [a.url for a in actions if a.is_remove]
 
             if add or remove:
-                log('Uploading: +%d / -%d', len(add), len(remove), sender=self)
+                logger.debug('Uploading: +%d / -%d', len(add), len(remove))
                 # Only do a push request if something has changed
                 result = self._client.update_subscriptions(self.device_id, add, remove)
 
@@ -512,26 +556,27 @@ class MygPoClient(object):
                 # Store URL rewrites for later retrieval by GUI
                 for old_url, new_url in result.update_urls:
                     if new_url:
-                        log('Rewritten URL: %s', new_url, sender=self)
+                        logger.debug('Rewritten URL: %s', new_url)
                         self._store.save(RewrittenUrl(old_url, new_url))
 
             # Actions have been uploaded to the server - remove them
             self._store.remove(actions)
-            log('All actions have been uploaded to the server.', sender=self)
+            logger.debug('All actions have been uploaded to the server.')
             return True
         except Exception, e:
-            log('Cannot upload subscriptions: %s', str(e), sender=self, traceback=True)
+            logger.error('Cannot upload subscriptions: %s', str(e), exc_info=True)
             return False
 
     def update_device(self, action):
         try:
-            log('Uploading device settings...', sender=self)
+            logger.debug('Uploading device settings...')
             self._client.update_device_settings(action.device_id, \
                     action.caption, action.device_type)
-            log('Device settings uploaded.', sender=self)
+            logger.debug('Device settings uploaded.')
             return True
         except Exception, e:
-            log('Cannot update device %s: %s', self.device_id, str(e), sender=self, traceback=True)
+            logger.error('Cannot update device %s: %s', self.device_id,
+                str(e), exc_info=True)
             return False
 
     def get_devices(self):
