@@ -1,7 +1,8 @@
 # -*- coding: utf-8 -*-
 #
 # gPodder - A media aggregator and podcast client
-# Copyright (c) 2005-2012 Thomas Perl and the gPodder Team
+# Copyright (c) 2005-2011 Thomas Perl and the gPodder Team
+# Copyright (c) 2011 Neal H. Walfield
 #
 # gPodder is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -30,7 +31,9 @@ are not tied to any specific part of gPodder.
 """
 
 import gpodder
-from gpodder.liblogger import log
+
+import logging
+logger = logging.getLogger(__name__)
 
 import os
 import os.path
@@ -40,6 +43,7 @@ import stat
 import shlex
 import socket
 import sys
+import string
 
 import re
 import subprocess
@@ -55,6 +59,7 @@ import urllib2
 import httplib
 import webbrowser
 import mimetypes
+import itertools
 
 import feedparser
 
@@ -66,7 +71,10 @@ N_ = gpodder.ngettext
 
 
 import locale
-locale.setlocale(locale.LC_ALL, '')
+try:
+    locale.setlocale(locale.LC_ALL, '')
+except Exception, e:
+    logger.warn('Cannot set locale (%s)', e, exc_info=True)
 
 # Native filesystem encoding detection
 encoding = sys.getfilesystemencoding()
@@ -75,8 +83,8 @@ if encoding is None:
     if 'LANG' in os.environ and '.' in os.environ['LANG']:
         lang = os.environ['LANG']
         (language, encoding) = lang.rsplit('.', 1)
-        log('Detected encoding: %s', encoding)
-    elif gpodder.ui.maemo:
+        logger.info('Detected encoding: %s', encoding)
+    elif gpodder.ui.fremantle:
         encoding = 'utf-8'
     elif gpodder.win32:
         # To quote http://docs.python.org/howto/unicode.html:
@@ -85,9 +93,23 @@ if encoding is None:
         encoding = 'mbcs'
     else:
         encoding = 'iso-8859-15'
-        log('Assuming encoding: ISO-8859-15 ($LANG not set).')
+        logger.info('Assuming encoding: ISO-8859-15 ($LANG not set).')
 
 
+# Filename / folder name sanitization
+def _sanitize_char(c):
+    if c in string.whitespace:
+        return ' '
+    elif c in ',-.()':
+        return c
+    elif c in string.punctuation or ord(c) <= 31:
+        return '_'
+
+    return c
+
+SANITIZATION_TABLE = ''.join(map(_sanitize_char, map(chr, range(256))))
+del _sanitize_char
+
 # Used by file_type_by_extension()
 _BUILTIN_FILE_TYPES = None
 
@@ -104,7 +126,7 @@ def make_directory( path):
     try:
         os.makedirs( path)
     except:
-        log( 'Could not create directory: %s', path)
+        logger.warn('Could not create directory: %s', path)
         return False
 
     return True
@@ -125,13 +147,25 @@ def normalize_feed_url(url):
     simply assume the user intends to add a http:// feed.
 
     >>> normalize_feed_url('curry.com')
-    'http://curry.com'
+    'http://curry.com/'
 
     There are even some more shortcuts for advanced users
     and lazy typists (see the source for details).
 
     >>> normalize_feed_url('fb:43FPodcast')
     'http://feeds.feedburner.com/43FPodcast'
+
+    It will also take care of converting the domain name to
+    all-lowercase (because domains are not case sensitive):
+
+    >>> normalize_feed_url('http://Example.COM/')
+    'http://example.com/'
+
+    Some other minimalistic changes are also taken care of,
+    e.g. a ? with an empty query is removed:
+
+    >>> normalize_feed_url('http://example.org/test?')
+    'http://example.org/test'
     """
     if not url or len(url) < 8:
         return None
@@ -158,21 +192,24 @@ def normalize_feed_url(url):
     if not '://' in url:
         url = 'http://' + url
 
-    # The scheme of the URL should be all-lowercase
-    (scheme, rest) = url.split('://', 1)
-    scheme = scheme.lower()
+    scheme, netloc, path, query, fragment = urlparse.urlsplit(url)
+
+    # Schemes and domain names are case insensitive
+    scheme, netloc = scheme.lower(), netloc.lower()
+
+    # Normalize empty paths to "/"
+    if path == '':
+        path = '/'
 
     # feed://, itpc:// and itms:// are really http://
     if scheme in ('feed', 'itpc', 'itms'):
         scheme = 'http'
 
-    # Re-assemble our URL
-    url = scheme + '://' + rest
+    if scheme not in ('http', 'https', 'ftp', 'file'):
+        return None
 
-    if scheme in ('http', 'https', 'ftp', 'file'):
-        return url
-
-    return None
+    # urlunsplit might return "a slighty different, but equivalent URL"
+    return urlparse.urlunsplit((scheme, netloc, path, query, fragment))
 
 
 def username_password_from_url(url):
@@ -238,14 +275,6 @@ def username_password_from_url(url):
     return (username, password)
 
 
-def directory_is_writable( path):
-    """
-    Returns True if the specified directory exists and is writable
-    by the current user.
-    """
-    return os.path.isdir( path) and os.access( path, os.W_OK)
-
-
 def calculate_size( path):
     """
     Tries to calculate the size of a directory, including any 
@@ -270,9 +299,9 @@ def calculate_size( path):
                 try:
                     sum += calculate_size(os.path.join(path, item))
                 except:
-                    log('Cannot get size for %s', path)
+                    logger.warn('Cannot get size for %s', path, exc_info=True)
         except:
-            log('Cannot access: %s', path)
+            logger.warn('Cannot access %s', path, exc_info=True)
 
         return sum
 
@@ -296,25 +325,10 @@ def file_modification_datetime(filename):
         timestamp = s[stat.ST_MTIME]
         return datetime.datetime.fromtimestamp(timestamp)
     except:
-        log('Cannot get modification timestamp for %s', filename)
+        logger.warn('Cannot get mtime for %s', filename, exc_info=True)
         return None
 
 
-def file_modification_timestamp(filename):
-    """
-    Returns the modification date of the specified file as a number
-    or -1 if the modification date cannot be determined.
-    """
-    if filename is None:
-        return -1
-    try:
-        s = os.stat(filename)
-        return s[stat.ST_MTIME]
-    except:
-        log('Cannot get modification timestamp for %s', filename)
-        return -1
-
-
 def file_age_in_days(filename):
     """
     Returns the age of the specified filename in days or
@@ -360,7 +374,7 @@ def get_free_disk_space_win32(path):
         userFree, userTotal, freeOnDisk = win32file.GetDiskFreeSpaceEx(drive)
         return userFree
     except ImportError:
-        log('Warning: Running on Win32 but win32api/win32file not installed.')
+        logger.warn('Running on Win32 but win32api/win32file not installed.')
 
     # Cannot determine free disk space
     return 0
@@ -383,21 +397,7 @@ def get_free_disk_space(path):
 
     s = os.statvfs(path)
 
-    free_space = s.f_bavail * s.f_bsize
-
-    if free_space == 0:
-        # Try to fallback to using GIO to determine free space
-        # This fixes issues with GVFS-mounted iPods (bug 1361)
-        try:
-            import gio
-            file = gio.File(path)
-            info = file.query_filesystem_info(gio.FILE_ATTRIBUTE_FILESYSTEM_FREE)
-            return info.get_attribute_uint64(gio.FILE_ATTRIBUTE_FILESYSTEM_FREE)
-        except Exception, e:
-            log('Free space is zero. Fallback using "gio" failed.', traceback=True)
-            return free_space
-
-    return free_space
+    return s.f_bavail * s.f_bsize
 
 
 def format_date(timestamp):
@@ -420,7 +420,7 @@ def format_date(timestamp):
     try:
         timestamp_date = time.localtime(timestamp)[:3]
     except ValueError, ve:
-        log('Warning: Cannot convert timestamp', traceback=True)
+        logger.warn('Cannot convert timestamp', exc_info=True)
         return None
     
     if timestamp_date == today:
@@ -431,7 +431,7 @@ def format_date(timestamp):
     try:
         diff = int( (time.time() - timestamp)/seconds_in_a_day )
     except:
-        log('Warning: Cannot convert "%s" to date.', timestamp, traceback=True)
+        logger.warn('Cannot convert "%s" to date.', timestamp, exc_info=True)
         return None
 
     try:
@@ -758,7 +758,7 @@ def object_string_formatter( s, **kwargs):
                     to_s = getattr( o, attr)
                     result = result.replace( from_s, to_s)
                 except:
-                    log( 'Could not replace attribute "%s" in string "%s".', attr, s)
+                    logger.warn('Could not replace attribute "%s" in string "%s".', attr, s)
 
     return result
 
@@ -911,64 +911,94 @@ def get_real_url(url):
     try:
         return urlopen(url).geturl()
     except:
-        log('Error getting real url for %s', url, traceback=True)
+        logger.error('Getting real url for %s', url, exc_info=True)
         return url
 
 
-def find_command( command):
+def find_command(command):
     """
     Searches the system's PATH for a specific command that is
     executable by the user. Returns the first occurence of an
-    executable binary in the PATH, or None if the command is 
+    executable binary in the PATH, or None if the command is
     not available.
+
+    On Windows, this also looks for "<command>.bat" and
+    "<command>.exe" files if "<command>" itself doesn't exist.
     """
 
     if 'PATH' not in os.environ:
         return None
 
-    for path in os.environ['PATH'].split( os.pathsep):
-        command_file = os.path.join( path, command)
-        if os.path.isfile( command_file) and os.access( command_file, os.X_OK):
+    for path in os.environ['PATH'].split(os.pathsep):
+        command_file = os.path.join(path, command)
+        if gpodder.win32 and not os.path.exists(command_file):
+            for extension in ('.bat', '.exe'):
+                cmd = command_file + extension
+                if os.path.isfile(cmd):
+                    command_file = cmd
+                    break
+        if os.path.isfile(command_file) and os.access(command_file, os.X_OK):
             return command_file
-        
+
     return None
 
-
-def http_get_and_gunzip(uri):
-    """
-    Does a HTTP GET request and tells the server that we accept
-    gzip-encoded data. This is necessary, because the Apple iTunes
-    server will always return gzip-encoded data, regardless of what
-    we really request.
-
-    Returns the uncompressed document at the given URI.
-    """
-    request = urllib2.Request(uri)
-    request.add_header("Accept-encoding", "gzip")
-    usock = urllib2.urlopen(request)
-    data = usock.read()
-    if usock.headers.get('content-encoding', None) == 'gzip':
-        data = gzip.GzipFile(fileobj=StringIO.StringIO(data)).read()
-    return data
-
+idle_add_handler = None
 
 def idle_add(func, *args):
-    """
-    This is a wrapper function that does the Right
-    Thing depending on if we are running a GTK+ GUI or
-    not. If not, we're simply calling the function.
+    """Run a function in the main GUI thread
 
-    If we are a GUI app, we use gobject.idle_add() to
-    call the function later - this is needed for
-    threads to be able to modify GTK+ widget data.
+    This is a wrapper function that does the Right Thing depending on if we are
+    running on Gtk+, Qt or CLI.
+
+    You should use this function if you are calling from a Python thread and
+    modify UI data, so that you make sure that the function is called as soon
+    as possible from the main UI thread.
     """
-    if gpodder.ui.desktop or gpodder.ui.maemo:
+    if gpodder.ui.gtk:
         import gobject
-        def x(f, *a):
-            f(*a)
-            return False
-
         gobject.idle_add(func, *args)
+    elif gpodder.ui.qt:
+        from PySide.QtCore import Signal, QTimer, QThread, Qt, QObject
+
+        class IdleAddHandler(QObject):
+            signal = Signal(object)
+            def __init__(self):
+                QObject.__init__(self)
+
+                self.main_thread_id = QThread.currentThreadId()
+
+                self.signal.connect(self.run_func)
+
+            def run_func(self, func):
+                assert QThread.currentThreadId() == self.main_thread_id, \
+                    ("Running in %s, not %s"
+                     % (str(QThread.currentThreadId()),
+                        str(self.main_thread_id)))
+                func()
+
+            def idle_add(self, func, *args):
+                def doit():
+                    try:
+                        func(*args)
+                    except Exception, e:
+                        logger.exception("Running %s%s: %s",
+                                         func, str(tuple(args)), str(e))
+
+                if QThread.currentThreadId() == self.main_thread_id:
+                    # If we emit the signal in the main thread,
+                    # then the function will be run immediately.
+                    # Instead, use a single shot timer with a 0
+                    # timeout: this will run the function when the
+                    # event loop next iterates.
+                    QTimer.singleShot(0, doit)
+                else:
+                    self.signal.emit(doit)
+
+        global idle_add_handler
+        if idle_add_handler is None:
+            idle_add_handler = IdleAddHandler()
+
+        idle_add_handler.idle_add(func, *args)
     else:
         func(*args)
 
@@ -978,29 +1008,12 @@ def bluetooth_available():
     Returns True or False depending on the availability
     of bluetooth functionality on the system.
     """
-    if gpodder.ui.maemo:
-        return True
-    elif find_command('bluetooth-sendto') or \
+    if find_command('bluetooth-sendto') or \
             find_command('gnome-obex-send'):
         return True
     else:
         return False
 
-def bluetooth_send_files_maemo(filenames):
-    """Maemo implementation of Bluetooth file transfer
-
-    Takes a list of (absolute and local) filenames that are
-    submitted to the Maemo Bluetooth UI for file transfer.
-
-    This method works in Diablo and also in Fremantle.
-    """
-    import dbus
-    bus = dbus.SystemBus()
-    o = bus.get_object('com.nokia.bt_ui', '/com/nokia/bt_ui', False)
-    i = dbus.Interface(o, 'com.nokia.bt_ui')
-    i.show_send_file_dlg(['file://'+f for f in filenames])
-    return True
-
 
 def bluetooth_send_file(filename):
     """
@@ -1020,7 +1033,7 @@ def bluetooth_send_file(filename):
         command_line.append(filename)
         return (subprocess.Popen(command_line).wait() == 0)
     else:
-        log('Cannot send file. Please install "bluetooth-sendto" or "gnome-obex-send".')
+        logger.error('Cannot send file. Please install "bluetooth-sendto" or "gnome-obex-send".')
         return False
 
 
@@ -1118,45 +1131,6 @@ def http_request(url, method='HEAD'):
     conn.request(method, url[start:])
     return conn.getresponse()
 
-def get_episode_info_from_url(url):
-    """
-    Try to get information about a podcast episode by sending
-    a HEAD request to the HTTP server and parsing the result.
-
-    The return value is a dict containing all fields that 
-    could be parsed from the URL. This currently contains:
-    
-      "length": The size of the file in bytes
-      "pubdate": The unix timestamp for the pubdate
-
-    If there is an error, this function returns {}. This will
-    only function with http:// and https:// URLs.
-    """
-    if not (url.startswith('http://') or url.startswith('https://')):
-        return {}
-
-    r = http_request(url)
-    result = {}
-
-    log('Trying to get metainfo for %s', url)
-
-    if 'content-length' in r.msg:
-        try:
-            length = int(r.msg['content-length'])
-            result['length'] = length
-        except ValueError, e:
-            log('Error converting content-length header.')
-
-    if 'last-modified' in r.msg:
-        try:
-            parsed_date = feedparser._parse_date(r.msg['last-modified'])
-            pubdate = time.mktime(parsed_date)
-            result['pubdate'] = pubdate
-        except:
-            log('Error converting last-modified header.')
-
-    return result
-
 
 def gui_open(filename):
     """
@@ -1168,41 +1142,15 @@ def gui_open(filename):
        on Maemo, osso is used to communicate with Nokia Media Player
     """
     try:
-        if gpodder.ui.maemo:
-            try:
-                import osso
-            except ImportError, ie:
-                log('Cannot import osso module on maemo.')
-                return False
-
-            log('Using Nokia Media Player to open %s', filename)
-            context = osso.Context('gPodder', gpodder.__version__, False)
-            filename = filename.encode('utf-8')
-
-            # Fix for Maemo bug 7162 (for local files with "#" in filename)
-            if filename.startswith('/'):
-                filename = 'file://' + urllib.quote(filename)
-
-            rpc = osso.Rpc(context)
-            app = 'mediaplayer'
-
-            _unneeded, extension = os.path.splitext(filename.lower())
-
-            # Fix for Maemo bug 5588 (use PDF viewer and images app)
-            if extension == '.pdf':
-                app = 'osso_pdfviewer'
-            elif extension in ('.jpg', '.jpeg', '.png'):
-                app = 'image_viewer'
-
-            svc, path = (x % app for x in ('com.nokia.%s', '/com/nokia/%s'))
-            rpc.rpc_run(svc, path, svc, 'mime_open', (filename,))
-        elif gpodder.win32:
+        if gpodder.win32:
             os.startfile(filename)
+        elif gpodder.osx:
+            subprocess.Popen(['open', filename])
         else:
             subprocess.Popen(['xdg-open', filename])
         return True
     except:
-        log('Cannot open file/folder: "%s"', filename, traceback=True)
+        logger.error('Cannot open file/folder: "%s"', filename, exc_info=True)
         return False
 
 
@@ -1212,7 +1160,7 @@ def open_website(url):
     browser. This uses Python's "webbrowser" module, so
     make sure your system is set up correctly.
     """
-    if gpodder.ui.maemo:
+    if gpodder.ui.fremantle:
         import osso
         context = osso.Context('gPodder', gpodder.__version__, False)
         rpc = osso.Rpc(context)
@@ -1222,35 +1170,6 @@ def open_website(url):
     else:
         threading.Thread(target=webbrowser.open, args=(url,)).start()
 
-def convert_bytes(d):
-    """
-    Convert byte strings to unicode strings
-
-    This function will decode byte strings into unicode
-    strings. Any other data types will be left alone.
-
-    >>> convert_bytes(None)
-    >>> convert_bytes(1)
-    1
-    >>> convert_bytes(4711L)
-    4711L
-    >>> convert_bytes(True)
-    True
-    >>> convert_bytes(3.1415)
-    3.1415
-    >>> convert_bytes('Hello')
-    u'Hello'
-    >>> convert_bytes(u'Hey')
-    u'Hey'
-    """
-    if d is None:
-        return d
-    if any(isinstance(d, t) for t in (int, long, bool, float)):
-        return d
-    elif not isinstance(d, unicode):
-        return d.decode('utf-8', 'ignore')
-    return d
-
 def sanitize_encoding(filename):
     r"""
     Generate a sanitized version of a string (i.e.
@@ -1278,20 +1197,19 @@ def sanitize_filename(filename, max_length=0, use_ascii=False):
     If use_ascii is True, don't encode in the native language,
     but use only characters from the ASCII character set.
     """
-    global encoding
-    if use_ascii:
-        e = 'ascii'
-    else:
-        e = encoding
-
     if not isinstance(filename, unicode):
         filename = filename.decode(encoding, 'ignore')
 
     if max_length > 0 and len(filename) > max_length:
-        log('Limiting file/folder name "%s" to %d characters.', filename, max_length)
+        logger.info('Limiting file/folder name "%s" to %d characters.',
+                filename, max_length)
         filename = filename[:max_length]
 
-    return re.sub('[/|?*<>:+\[\]\"\\\]', '_', filename.strip().encode(e, 'ignore'))
+    filename = filename.encode('ascii' if use_ascii else encoding, 'ignore')
+    filename = filename.translate(SANITIZATION_TABLE)
+    filename = filename.strip('.' + string.whitespace)
+
+    return filename
 
 
 def find_mount_point(directory):
@@ -1392,19 +1310,6 @@ def isabs(string):
     if protocolPattern.match(string): return 1
     return os.path.isabs(string)
 
-def rel2abs(path, base = os.curdir):
-    """ converts a relative path to an absolute path.
-
-    @param path the path to convert - if already absolute, is returned
-    without conversion.
-    @param base - optional. Defaults to the current directory.
-    The base is intelligently concatenated to the given relative path.
-    @return the relative path of path from base
-    Source: http://code.activestate.com/recipes/208993/
-    """
-    if isabs(path): return path
-    retval = os.path.join(base,path)
-    return os.path.abspath(retval)
 
 def commonpath(l1, l2, common=[]):
     """
@@ -1434,32 +1339,6 @@ def relpath(p1, p2):
     return os.path.join(*p)
 
 
-def run_external_command(command_line):
-    """
-    This is the function that will be called in a separate
-    thread that will call an external command (specified by
-    command_line). In case of problem (i.e. the command has
-    not been found or there has been another error), we will
-    call the notification function with two arguments - the
-    first being the error message and the second being the
-    title to be used for the error message.
-    """
-
-    def open_process(command_line):
-        log('Running external command: %s', command_line)
-        p = subprocess.Popen(command_line, shell=True)
-        result = p.wait()
-        if result == 127:
-            log('Command not found: %s', command_line)
-        elif result == 126:
-            log('Command permission denied: %s', command_line)
-        elif result > 0:
-            log('Command returned an error (%d): %s', result, command_line)
-        else:
-            log('Command finished successfully: %s', command_line)
-
-    threading.Thread(target=open_process, args=(command_line,)).start()
-
 def get_hostname():
     """Return the hostname of this computer
 
@@ -1483,7 +1362,7 @@ def detect_device_type():
     Possible return values:
     desktop, laptop, mobile, server, other
     """
-    if gpodder.ui.maemo:
+    if gpodder.ui.fremantle:
         return 'mobile'
     elif glob.glob('/proc/acpi/battery/*'):
         # Linux: If we have a battery, assume Laptop
@@ -1527,3 +1406,32 @@ def write_m3u_playlist(m3u_filename, episodes, extm3u=True):
 
     f.close()
 
+
+def generate_names(filename):
+    basename, ext = os.path.splitext(filename)
+    for i in itertools.count():
+        if i:
+            yield '%s (%d)%s' % (basename, i+1, ext)
+        else:
+            yield filename
+
+
+def is_known_redirecter(url):
+    """Check if a URL redirect is expected, and no filenames should be updated
+
+    We usually honor URL redirects, and update filenames accordingly.
+    In some cases (e.g. Soundcloud) this results in a worse filename,
+    so we hardcode and detect these cases here to avoid renaming files
+    for which we know that a "known good default" exists.
+
+    The problem here is that by comparing the currently-assigned filename
+    with the new filename determined by the URL, we cannot really determine
+    which one is the "better" URL (e.g. "n5rMSpXrqmR9.128.mp3" for Soundcloud).
+    """
+
+    # Soundcloud-hosted media downloads (we take the track name as filename)
+    if url.startswith('http://ak-media.soundcloud.com/'):
+        return True
+
+    return False
+
